<?php
/**
 * @file
 * Kazoo API Query class.
 */


// A base class for Kazoo API query classes.
abstract class kazoo_api_base extends rest_api_query {
  protected $reexecuting = FALSE;

  // Kazoo API expects an accept header.
  public $headers = array(
    'Content-type' => 'application/json',
    'Accept' => 'application/json',
  );

  // Kazoo API does not support file extensions, e.g. ".json".
  public $format = '';

  function __construct($url = NULL) {
    $this->access_point = $url ? $url : kazoo_var('access_point');
    $this->initialized = TRUE;
  }

  function execute() {
    // Kazoo API expects all data parameters to be on the "data" property.
    if (!empty($this->data) && !$this->reexecuting) {
      $this->data = array('data' => $this->data);
    }
    return parent::execute();
  }
}

// A query class just for checking credentials and/or getting a new Auth token.
class kazoo_api_auth extends kazoo_api_base {
  public $method = 'PUT';
  public $resource = 'user_auth';

  // Disable caching on authentication-queries.
  public $cache_table = FALSE;

  function __construct($hash = NULL, $account_name = NULL, $url = NULL) {
    $this->data['credentials'] = $hash ? $hash : kazoo_var('hash');
    $this->data['account_name'] = $account_name ? $account_name : kazoo_var('account_name');
    parent::__construct($url);
  }
}

// A query class for most queries.
class kazoo_api_query extends kazoo_api_base {
  // Disable caching when running with simpletest.
  // public $cache_table = FALSE;

  function __construct() {
    // Most queries need the Auth Token header and the account ID.
    $this->headers['X-Auth-Token'] = kazoo_var('token');
    $this->identifier = kazoo_var('account_id');
    parent::__construct();
  }

  function execute() {
    $return = parent::execute();

    // Get a new auth token if it has expired.
    if ($this->code == 401 && !$this->reexecuting) {
      $this->reexecuting = TRUE;
      return $this->authenticate();
    }

    // Set the object's own success, error, code and result properties from the
    // Kazoo API's response data.
    $this->success = (isset($this->result->status) && $this->result->status === 'success');
    if (!empty($this->result)) {
      if ($this->success) {
        // The Kazoo API sets "error" messages like "Created" on success.
        $this->error = NULL;
      }
      else {
        $this->error = $this->result->message;
        $this->code = $this->result->error;
      }
      $this->result = $this->result->data;
    }

    return $return;
  }

  // Gets a new valid authentication token and re-executes the query with it.
  function authenticate() {
    $auth_query = new kazoo_api_auth();
    $auth_query->execute();
    if ($auth_query->success) {
      // Execute the query again with the new header.
      $this->headers['X-Auth-Token'] = $auth_query->result->auth_token;
      $id = $auth_query->result->data->account_id;
      if (empty($this->identifier)) {
        $this->identifier = $id;
      }

      // Prevent recursing more than once.
      $return = $this->execute();

      // Save the new auth token if it worked.
      if ($this->success) {
        variable_set('kazoo_api_token', $this->headers['X-Auth-Token']);
        variable_set('kazoo_api_account_id', $id);
      }
      return $return;
    }
  }
}
